/************************************************************************************
*                                                                                   *
*   File Name   : MAIN_SU.c                                                         *
*   Contents    : Main Loop for Sanduhr                                             *
*   Version     : 1.19                                                               *
*************************************************************************************/ 

#include "globals.h"
#include "led_disp_drv.h"

extern unsigned int mk_month;
extern unsigned int mk_day;

unsigned long UL_TIRQ_count1ms = 0;
unsigned long UL_int2_IRQ_timestamp = 0;
unsigned int  UI_STATE;
unsigned int  UI_XDIR_VOLTAGE_AD;
unsigned char UC_CURR_PAGE;
unsigned char UC_MINS_TO_RUN;
unsigned char UC_CURRENT_POSITION = D_POS_UNDEF;
volatile char C_ENC_DELTA = 0;
_Bool         B_SWITCH_OK = D_FALSE;

void main (void)
{
unsigned char data[8];
unsigned int  uc_max_page;
_Bool         b_valid_acceleration = D_FALSE;

    //start initialization of the uC
    f_vd_controllerInit();

    /*-----------------------------------
    -   Loop of main                    -
    -----------------------------------*/
    while(1)
    {
	//Calculations
        //encoder input to minutes to run conversion
	if (C_ENC_DELTA < 0)
	{
	   UC_MINS_TO_RUN =  (-1 * C_ENC_DELTA)/2;
	}
	else
	{
	   UC_MINS_TO_RUN =  C_ENC_DELTA/2;
	}	
	if (UC_MINS_TO_RUN > 99)
	{
	   UC_MINS_TO_RUN = 99;
	}
        //check if all needed acceleration values are really acquired
        if ( (UI_XDIR_VOLTAGE_AD > D_lowestAccelerationAcquisitionLevel) &&
             (UI_XDIR_VOLTAGE_AD < 1024)
           )
        {
           b_valid_acceleration = D_TRUE;
        }

        //check direction if valid acceleration values are acquired
        if (b_valid_acceleration == D_TRUE)
        {
           if (UI_XDIR_VOLTAGE_AD > D_uprightLevelXDir)
           {
              UC_CURRENT_POSITION = D_POS_UPRIGHT;
           }
	   else if (UI_XDIR_VOLTAGE_AD < D_headfirstLevelXDir)
           {
	           UC_CURRENT_POSITION = D_POS_HEADFIRST;        
	   }
           else
           {
	      UC_CURRENT_POSITION = D_POS_MOVING;        
	   }
        } //end of if (b_valid_acceleration == D_TRUE)
        else
        {
           //if there is no complete valid acquisition we assume upright position
           UC_CURRENT_POSITION = D_POS_UPRIGHT;
        }
        
        //Statemachine
        switch (UI_STATE)
	{
          case D_undefState:
	       UI_STATE = D_revDisplayState;
               break;
          case D_revDisplayState:
               if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
               {
                  f_vd_SHOW_NUMBERS((unsigned char)mk_month, (unsigned char)mk_day, D_4digits_upright_disp );
               }
               else if (UC_CURRENT_POSITION == D_POS_HEADFIRST)
               {
                  f_vd_SHOW_NUMBERS((unsigned char)mk_month, (unsigned char)mk_day, D_4digits_headfirst_disp );
               }
               else
               {
                  f_vd_SHOW_SANDUHR_PAGE(61, D_upright_sand_disp); //show straight bar
               }
               //t4
               if (UL_TIRQ_count1ms >= D_SHOW_REV_TIME_MS) //leave showRevState after a defined amount of time
               {
                   UI_STATE = D_adjustTimeState;
               }			
               break;		
          case D_adjustTimeState:
               UC_CURR_PAGE = 0;
               if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
               {
	          f_vd_SHOW_NUMBERS(UC_MINS_TO_RUN, 0, D_2digits_upright_disp );
               }
               else if (UC_CURRENT_POSITION == D_POS_HEADFIRST)
               {
                  f_vd_SHOW_NUMBERS(UC_MINS_TO_RUN, 0, D_2digits_headfirst_disp );
               }
               else
               {
                  f_vd_SHOW_SANDUHR_PAGE(61, D_upright_sand_disp); //show straight bar
               }
               //t8
               if (B_SWITCH_OK == D_TRUE)
               {  
                  B_SWITCH_OK = D_FALSE;    
                  if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
                  {
                     UC_CURR_PAGE = 0;
                  }
                  else if (UC_CURRENT_POSITION == D_POS_HEADFIRST)
                  {
                     UC_CURR_PAGE = 60;
                  }
		  UI_STATE = D_runningState;
               }
	       break;
          case D_runningState:
               //t12
               if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
               {
                  uc_max_page = 60;
               }
               else if (UC_CURRENT_POSITION == D_POS_HEADFIRST)
               {
                  uc_max_page = 0;
               }
               if (UC_CURR_PAGE == uc_max_page)
               {
                  UI_STATE = D_readyState;
               }
               else
               {
                  if (UC_CURRENT_POSITION == D_POS_MOVING)
                  {
                     f_vd_SHOW_SANDUHR_PAGE(61, D_upright_sand_disp); //show straight bar
                  }
                  else
                  {
                     if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
                     {
                        f_vd_SHOW_SANDUHR_PAGE(UC_CURR_PAGE, D_upright_sand_disp); //Upright
                     }
                     else
                     {
                        f_vd_SHOW_SANDUHR_PAGE(UC_CURR_PAGE, D_headfirst_sand_disp); //Headfirst
                     }
                  }
               }
	       break;
          case D_readyState:
               if (UC_CURRENT_POSITION == D_POS_UPRIGHT)
               {
                  f_vd_SHOW_SANDUHR_PAGE(60, D_upright_sand_disp); //Upright
               }
               else if (UC_CURRENT_POSITION == D_POS_HEADFIRST)
               {
                  f_vd_SHOW_SANDUHR_PAGE(0, D_headfirst_sand_disp); //Headfirst
               }
               tzs   = 1; //Start Timer Z = Buzzer On
               //t14
               if (B_SWITCH_OK == D_TRUE)
               {  
                  B_SWITCH_OK = D_FALSE;              
                  tzs   = 0; //Stop Timer Z = Buzzer Off
		  UI_STATE = D_adjustTimeState;
               }
               break;
        } //end of switch (UI_STATE) 
    } //end of while(1)
} //end of void main (void)
